package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Vector;

import org.jfree.data.category.DefaultCategoryDataset;

import edu.ucla.cs.dbevolution.ApplicationParameters;

/**
 * Create and execute the statistic that performs an analysis over the database, retrieving if there are tables that over
 * their lifetime are evolved and have been rolled back to a previous version 
 *	@author fangar
 */
public class RolledBackTables extends Statistic implements StatisticRisk{

	private HashMap<String, Integer> tableRollbacks;
	
	/**
	 * TODO
	 * Set the default filed for the statistic
	 */
	public RolledBackTables() {
		setTitle("Rolledback tables");
		setStatDescription("The statistic analyze the evolution's story of the table, looking into possible situations of rollbacks; " +
				"a \"rollback\" occurs when a situation like the one described here occurs: at the time n-1 the table has a structure x; then at the time n, the table is modified by the database administrator" +
				"changing his structure; the new structure is called y; the database evolvs and the db admin decide, at the time n+1, to change again the structure of the table into x;" +
				"the table has undergone a rollback changing its structure from x to y and then back to x again");
		setSqlStmt("The sql analyze the structure, in terms of columns, of the table during each revision comparing it with the structure of the revision + 1; creates a checkpoint every change of structure and then analyze each checkpoint" +
				"looking for possible rollback situations");
		setRange("rollbacks");
		setDomain("Tables");
		setStatName("statRolledbackTables");
		setChartOrientation("h");
		setChartType("bar");
		setTitlePage("Statistic: Rolledback Tables");
		setStatPage(ApplicationParameters.getDbbasename() + getStatName() + ".html");
		setStatGroup("tables");
	}
	
	
	/**
	 * TODO
	 * @throws SQLException 
	 * 
	 */
	public void execute() throws SQLException{
		setDs(getTablesRolledBack());
		executeStatistic(this);
	}
	
	
	/**
	 * @param table
	 * @return
	 * @throws SQLException 
	 */
	private DefaultCategoryDataset getTablesRolledBack() throws SQLException{

		int rollbackCount;
		
		//dataset
		DefaultCategoryDataset ds = new DefaultCategoryDataset();
		
		//mapping of the table with its number of rollback
		HashMap<String, Integer> tableRollbacks = new HashMap<String, Integer>();
		
		Vector<String> tables = getDb().getTables(ApplicationParameters.getDbbasename()+"%");

		if(tables!=null){

			//create the CSV file and add the header to the file
			createCSVFile(this);
			addValueToCsv(this, new String[]{"TableName","Rollbacks"});
			
			for(String table:tables){
				rollbackCount = 0;

				ArrayList<Savepoint> points = null;
				Savepoint sPrev = null;


				try {
					System.out.println("ROLLBACK - PERFORMING TABLE: " + table);

					points = new ArrayList<Savepoint>();
					sPrev = null;


					String sql = "SELECT table_schema,column_name FROM " + ApplicationParameters.getHeaderEvolutionTables() + "COLUMNS c, " + ApplicationParameters.getEvolutionTable() + " d WHERE c.table_schema = d.schema_unique_name " +
					"AND table_schema like ? AND table_name = ? ORDER BY d.tstart, table_schema, column_name";
					String[] params = new String[]{ApplicationParameters.getDbbasename() + "%", table};

					Vector<String[]> result = getDb().executeStatement(sql, params);
					
					if(result!=null){

						Vector<String> columnsName = new Vector<String>();


						String db = ( (String[]) result.get(0) )[0];
						String col = ( (String[]) result.get(0) )[1];

						columnsName.add(col);

						for(int i=1;i<result.size(); i++){

							String dbAc = ( (String[]) result.get(i) )[0]; 
							String colAc = ( (String[]) result.get(i) )[1];

							if(dbAc.equals(db)){
								columnsName.add(colAc);
							}
							else{
								Savepoint s = new Savepoint();
								s.setColumnsCount(columnsName.size());
								s.setColumnsName(columnsName);
								//points.add(s);

								if(sPrev!=null){
									if(sPrev.getColumnsCount()!=s.getColumnsCount()){
										sPrev = s;
										points.add(s);
									}
								}
								else{
									sPrev = s;
								}
								columnsName = new Vector<String>();
								columnsName.add(colAc);

								db = dbAc;
							}
						}
					}

					//search for a rolled back situation
					rollbackCount += checkTableRolledBack(points);
				}catch (SQLException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
				//the analysis of the table ended, write the number of rollnack into the csv file

				//add the values to the hashmap
				tableRollbacks.put(table, new Integer(rollbackCount));
				
				if(rollbackCount!=0){
					//add values to the dataset only if different from zero
					//avoid filling the table with lots of zero values
					addValueToDataset(ds, rollbackCount, "rollbacks", table);
				}
				
				//add the value to the csv
				//the csv file is complet, contains also the zero rolled back tables
				addValueToCsv(this, new String[]{table,String.valueOf(rollbackCount)});	
				
			}
		}
		//set the mapping to the local one
		setTableRollbacks(tableRollbacks);
		
		return ds;
	}





	/**
	 * @param points
	 */
	private int checkTableRolledBack(ArrayList<Savepoint> points) {

		int rollbackCount = 0;
		for(int countPoints = points.size()-1;countPoints>=0;countPoints--){
			Savepoint s = (Savepoint)points.get(countPoints);
			//compare savepoint with previous ones looking for rolledback situation
			for(int countPointsPrev =  0 + countPoints-1;countPointsPrev>=0;countPointsPrev--){
				if(s.compareSavepoints(s, points.get(countPointsPrev))){
					rollbackCount++;
				}
			}
		}
		return rollbackCount;

	}



	/**
	 * TODO
	 * @param tableName
	 * @return
	 */
	public int getRiskMapping(String tableName) {
		try{
			return tableRollbacks.get(tableName);
		}
		catch(NullPointerException nEx){
			return 0;
		}
	}



	public void setTableRollbacks(HashMap<String, Integer> tableRollbacks) {
		this.tableRollbacks = tableRollbacks;
	}





}
