package edu.ucla.cs.statistics;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.SQLException;
import java.util.Vector;

import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.DbOperations;
import edu.ucla.cs.dbevolution.Utils;

/**
 * The class contain all the data related to a statistic:
 * the link to the image, the link to the file with the dataset in CSV, the description 
 * and the title of the statistic etc..
 * @author fangar
 */
public class Statistic {


	
	private DbOperations db;
	
	public Statistic(){
		this.db = new DbOperations();
	}
	
	
	//**********************************//
	//		HTML PAGE DATA				//
	//**********************************//
	
	private String title;
	private String statDescription;
	private String imagePath;
	private String statName;
	private String titlePage;
	private String sqlStmt;
	private String linkData;
	private String imageAlt;
	private String statPage;
	private String statGroup;
	private String csvData;
	private File csvFile;
	
	//**********************************//
	//			CHART DATA				//
	//**********************************//
	
	private String chartOrientation;
	private DefaultCategoryDataset categoryDataset;
	private IntervalXYDataset intervalDataset;
	private XYSeriesCollection xyDataset;
	private NumberAxis axis;
	
	private String range;
	private String domain;
	private String chartType;
	

	

	
	/**
	 * Create a dataset for the given statistic
	 * @param st the statistic
	 * @return the dataset of the statistic
	 */
	protected DefaultCategoryDataset createCategoryDataset(Statistic st,Vector<StatisticValue> values){
		
		int countDomain=0;
		DefaultCategoryDataset ds = new DefaultCategoryDataset();
		//create the csv file and write the headers
		createCSVFile(st);
		addValueToCsv(st, new String[]{"DomainIndex","RealDomain","RangeValue"});
		
		for (StatisticValue stat : values) {
			if(st.getChartOrientation().equals("v")){
				//use as category a progressive number 
				//one series
				addValueToDataset(ds, stat.getY(), "", String.valueOf(countDomain));
			}
			else{
				//one series (rowKey)
				//multiple categories (table name)
				addValueToDataset(ds, stat.getY(), "",stat.getX());
			}
			countDomain++;
			
			//add value to csv
			addValueToCsv(st, new String[]{String.valueOf(countDomain),stat.getX(),String.valueOf(stat.getY())});
		}
		return  ds;
	}
	
	/**
	 * Create a dataset of type XYSeriesCollection and the csv file related to the dataset
	 * @param st the statistic
	 * @return the created dataset
	 * @throws SQLException 
	 */
	public XYSeriesCollection createXYDataset(Statistic st) throws SQLException{
		
		int countDomain=1;
		
		XYSeries series1 = new XYSeries("First");
		
		//create the csv file and write the headers
		createCSVFile(st);
		addValueToCsv(st, new String[]{"DomainIndex","RealDomain","RangeValue"});
		
		Vector<StatisticValue> values = db.GetStatCount(ApplicationParameters.getDbbasename(),st.getSqlStmt());
		for (StatisticValue stat : values) {
			series1.add(countDomain,stat.getY());
			countDomain++;
			
			//add value to csv
			addValueToCsv(st, new String[]{String.valueOf(countDomain),stat.getX(),String.valueOf(stat.getY())});
		}
		XYSeriesCollection dataset = new XYSeriesCollection();
		dataset.addSeries(series1);
		
		return dataset;
	}
	
	/**
	 * Add a given value to the dataset
	 * @param ds the dataset
	 * @param value the value to add
	 * @param rowKey 
	 * @param columnKey
	 */
	protected void addValueToDataset(DefaultCategoryDataset ds, Number value, String rowKey, String columnKey){
		ds.addValue(value, rowKey, columnKey);
	}

	
	
	/**
	 * Create the csv file for the selected statistic
	 * The file is created in the directory configured by the user in the configuration file
	 * @param stat the statistic to create the csv file for
	 */
	protected void createCSVFile(Statistic stat){

		File f = null;
		FileWriter fw = null;
		
		try {
			//set the variable to output the csv file
	
			if(Utils.checkCreateDir(StatisticsParameters.getStatPagePath()) && Utils.checkCreateDir(StatisticsParameters.getStatCsvPath())){
				f = new File(StatisticsParameters.getStatCsvPath() + ApplicationParameters.getDbbasename() + stat.getStatName() + ".csv");
				stat.setCsvData(f.getPath().replaceFirst(StatisticsParameters.getStatPagePath(), ""));
				stat.setCsvFile(f);
			}
			//open the file to create a new file if a file with the same name already exist
			fw = new FileWriter(f);
			fw.close();
		}
		catch (FileNotFoundException fEx) {
			System.out.println("File not found");
			return;
		}
		catch (IOException e) {
			System.out.println("Unable to create the csv file: " + e.getMessage());
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Add the given value to the csv file
	 * @param stat
	 * @param data
	 */
	protected void addValueToCsv(Statistic stat,String[] data){
		
		File f = null;
		
		//get a reference to the statistic csv file
		f = stat.getCsvFile();

		FileWriter fw = null;
		BufferedWriter bwr = null;
		
		try {
			fw = new FileWriter(f,true);
			
			//write the header to the csv file
			bwr = new BufferedWriter(fw);
			
			for(int count=0;count<data.length;count++){
				bwr.write(data[count].toString());
				bwr.write(";");
			}	
			bwr.write("\n");
			
			bwr.close();
			fw.close();
		
		} catch (IOException e) {
			System.out.println("ADD VALUE: Unable to create the csv file: " + e.getMessage());
			//e.printStackTrace();
		}
		
	}
	
	
	
	/**
	 * Execute all the statistics, either configured into the list xml file, either the one provided in the application
	 * @param stat the statistic to execute
	 * @throws SQLException 
	 */
	public void executeStatistic(Statistic stat) throws SQLException{
		
		boolean executed = true;
		//create the chart class and html class
		StatisticChart chart = new StatisticChart();
		
		System.out.println("Executing statistic " + stat.getStatName() + " .....");
		
		//set the data of the chart
		chart.setChartData(stat.getTitle(), stat.getRange(), stat.getDomain());
		chart.setChartName(stat.getStatName());
		
		//set the orientation of the chart
		if(stat.getChartOrientation().startsWith("v")){
			chart.setOrientation(PlotOrientation.VERTICAL);
		}
		else if(stat.getChartOrientation().startsWith("h")){
			chart.setOrientation(PlotOrientation.HORIZONTAL);
		}
		
		//create the chart relative to the statistic
		try{
			if(stat.getChartType().equalsIgnoreCase("line")){
				if(stat.getDs()==null){
					XYSeriesCollection ds = this.createXYDataset(stat);
					if(ds.getSeries(0).getItemCount() != 0){
						stat.setXyDataset(ds);
					}
					else{
						executed = false;
						this.setImageAlt("No data available for this statistics");
					}
					//stat.setXyDataset(this.createXYDataset(stat));
				}
				chart.setXyDataset(stat.getXyDataset());
				if(executed){
					chart.createXYLineChart();
				}
			}
			else if(stat.getChartType().equalsIgnoreCase("bar")){
				
				if(stat.getDs()==null){
					Vector<StatisticValue> values = db.GetStatCount(ApplicationParameters.getDbbasename(), stat.getSqlStmt());
					stat.setDs(this.createCategoryDataset(stat,values));
				}
				chart.setDataset(stat.getDs());

				/*
				if(chart.getOrientation() == PlotOrientation.VERTICAL){
					//override the width
					chart.setWidthDimension(stat.getDs().getColumnCount());					
				}
				else{
					//override the height
					chart.setHeightDimension(stat.getDs().getColumnCount());					
				}*/
				chart.setColumnsCount(stat.getDs().getColumnCount());
				chart.createBarChart();
			}
			else if(stat.getChartType().equalsIgnoreCase("barxy")){
				chart.setIntervalDataset(stat.getIntervalDataset());
				chart.createXyBarChart();			
			}
			
		}
		catch (IOException ie) {
			System.out.println("An error occured during the creation of the chart: " + ie.getMessage());
			System.exit(1);
		}

		//set statistic parameters to create the html page
		stat.setStatPage(ApplicationParameters.getDbbasename() + stat.getStatName() + StatisticsParameters.getPageExt());
		if(executed){
			stat.setImagePath(chart.getPathToChart().replaceFirst(StatisticsParameters.getStatPagePath(), ""));
		}
	}
	
	
	/**
	 * @return true if the statistic is of type lifetime
	 */
	public boolean isLifetimeCategory(){
		return false;
	}
	
	//**********************************//
	//		GETTER SETTER				//
	//**********************************//
	
	
	public String getStatName() {
		return statName;
	}
	public void setStatName(String statName) {
		this.statName = statName;
	}
	public String getStatDescription() {
		return statDescription;
	}
	public void setStatDescription(String description) {
		this.statDescription = description;
	}
	public String getImagePath() {
		return imagePath;
	}
	public void setImagePath(String image) {
		this.imagePath = image;
	}
	public String getTitle() {
		return title;
	}
	public void setTitle(String title) {
		this.title = title;
	}
	public String getTitlePage() {
		return titlePage;
	}
	public void setTitlePage(String titlePage) {
		this.titlePage = titlePage;
	}
	public String getSqlStmt() {
		return sqlStmt;
	}
	public void setSqlStmt(String sqlStmt) {
		this.sqlStmt = sqlStmt;
	}
	public String getLinkData() {
		return linkData;
	}
	public void setLinkData(String linkData) {
		this.linkData = linkData;
	}
	public String getImageAlt() {
		return imageAlt;
	}
	public void setImageAlt(String imageAlt) {
		this.imageAlt = imageAlt;
	}
	public String getStatPage() {
		return statPage;
	}
	public void setStatPage(String statPage) {
		this.statPage = statPage;
	}
	public String getRange() {
		return range;
	}
	public void setRange(String range) {
		this.range = range;
	}
	public String getDomain() {
		return domain;
	}
	public void setDomain(String domain) {
		this.domain = domain;
	}
	public String getChartType() {
		return chartType;
	}
	public void setChartType(String chartType) {
		this.chartType = chartType;
	}
	public String getChartOrientation() {
		return chartOrientation;
	}
	public void setChartOrientation(String chartOrientation) {
		this.chartOrientation = chartOrientation;
	}
	public DefaultCategoryDataset getDs() {
		return categoryDataset;
	}
	public void setDs(DefaultCategoryDataset ds) {
		this.categoryDataset = ds;
	}
	public NumberAxis getAxis() {
		return axis;
	}
	public void setAxis(NumberAxis axis) {
		this.axis = axis;
	}
	public String getCsvData() {
		return csvData;
	}
	public void setCsvData(String csvData) {
		this.csvData = csvData;
	}
	public String getStatGroup() {
		return statGroup;
	}
	public void setStatGroup(String statGroup) {
		this.statGroup = statGroup;
	}
	public IntervalXYDataset getIntervalDataset() {
		return intervalDataset;
	}
	public void setIntervalDataset(IntervalXYDataset  intervalDatasetDs) {
		this.intervalDataset = intervalDatasetDs;
	}
	public XYSeriesCollection getXyDataset() {
		return xyDataset;
	}
	public void setXyDataset(XYSeriesCollection xyDataset) {
		this.xyDataset = xyDataset;
	}
	public File getCsvFile() {
		return csvFile;
	}
	public void setCsvFile(File csvFile) {
		this.csvFile = csvFile;
	}
	public void setDb(DbOperations db) {
		this.db = db;
	}
	public DbOperations getDb() {
		return db;
	}

	
	
	
	
	
	
}
