/**
 * Copyright (C) 2007 Giorgio Orsi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
package it.polimi.elet.cautils.configurator;

import it.polimi.elet.cautils.configurator.exceptions.ReservedOptionException;
import jargs.gnu.CmdLineParser;
import jargs.gnu.CmdLineParser.Option;

import java.util.HashMap;
import java.util.Iterator;

/**
 * @author Giorgio Orsi (orsi@elet.polimi.it)
 * @version 0.1
 */
public class CACmdLineParser {
    private CmdLineParser parser;
    private HashMap<String,Option> params;
    private static boolean existing;
    
    /**
     * @return The unique instance of a command line parser
     * @throws InstantiationException If an instance of a command line parser
     *                                already exists.
     */
    public static CACmdLineParser getInstance() throws InstantiationException{
	if (!existing) {
	    return (new CACmdLineParser());
	}
	throw new InstantiationException("Only one parser at time is allowed");
    }
    
    /**
     * Generates a new command line parser.
     */
    private CACmdLineParser () {
	parser = new CmdLineParser();
	params = new HashMap<String,Option>();
    }
    
    /**
     * Adds a new boolean option to the command line
     * @param shortName The short name for the parameter {es: -o}
     * @param longName The long name for the parameter {es: --output}
     * @throws ReservedOptionException If someone tries to add a --config option
     */
    public void addBooleanParameter (char shortName, String longName) throws ReservedOptionException{
	if (longName.equalsIgnoreCase("config")) {
	    throw new ReservedOptionException("The keyword 'config' is reserved");
	}
	Option opt = parser.addBooleanOption(shortName, longName);
	params.put(longName, opt);
    }
    
    /**
     * Adds a new boolean option to the command line
     * @param longName The long name for the parameter {es: --output}
     * @throws ReservedOptionException If someone tries to add a --config option
     */
    public void addBooleanParameter (String longName) throws ReservedOptionException{
	if (longName.equalsIgnoreCase("config")) {
	    throw new ReservedOptionException("The keyword 'config' is reserved");
	}
	Option opt = parser.addBooleanOption(longName);
	params.put(longName, opt);
    }
      
    /**
     * Adds a new integer option to the command line
     * @param shortName The short name for the parameter {es: -o}
     * @param longName The long name for the parameter {es: --output}
     * @throws ReservedOptionException If someone tries to add a --config option
     */
    public void addIntegerParameter (char shortName, String longName) throws ReservedOptionException{
	if (longName.equalsIgnoreCase("config")) {
	    throw new ReservedOptionException("The keyword 'config' is reserved");
	}
	Option opt = parser.addIntegerOption(shortName, longName);
	params.put(longName, opt);
    }
    
    /**
     * Adds a new integer option to the command line
     * @param longName The short name for the parameter {es: --output}
     * @throws ReservedOptionException If someone tries to add a --config option
     */
    public void addIntegerParameter (String longName) throws ReservedOptionException{
	if (longName.equalsIgnoreCase("config")) {
	    throw new ReservedOptionException("The keyword 'config' is reserved");
	}
	Option opt = parser.addIntegerOption(longName);
	params.put(longName, opt);
    }
    
    /**
     * Adds a new string option to the command line
     * @param shortName The short name for the parameter {es: -o}
     * @param longName The long name for the parameter {es: --output}
     * @throws ReservedOptionException If someone tries to add a --config option
     */
    public void addStringParameter (char shortName, String longName) throws ReservedOptionException{
	if (longName.equalsIgnoreCase("config")) {
	    throw new ReservedOptionException("The keyword 'config' is reserved");
	}
	Option opt = parser.addStringOption(shortName, longName);
	params.put(longName, opt);
    }
    
    /**
     * Adds a new string option to the command line
     * @param longName The long name for the parameter {es: --output}
     * @throws ReservedOptionException If someone tries to add a --config option
     */
    public void addStringParameter (String longName) throws ReservedOptionException{
	if (longName.equalsIgnoreCase("config")) {
	    throw new ReservedOptionException("The keyword 'config' is reserved");
	}
	Option opt = parser.addStringOption(longName);
	params.put(longName, opt);
    }
    
    /**
     * Adds a new double option to the command line
     * @param shortName The short name for the parameter {es: -o}
     * @param longName The long name for the parameter {es: --output}
     * @throws ReservedOptionException If someone tries to add a --config option
     */
    public void addDoubleParameter (char shortName, String longName) throws ReservedOptionException{
	if (longName.equalsIgnoreCase("config")) {
	    throw new ReservedOptionException("The keyword 'config' is reserved");
	}
	Option opt = parser.addDoubleOption(shortName, longName);
	params.put(longName, opt);
    }
    
    /**
     * Adds a new double option to the command line
     * @param longName The long name for the parameter {es: --output}
     * @throws ReservedOptionException If someone tries to add a --config option
     */
    public void addDoubleParameter (String longName) throws ReservedOptionException{
	if (longName.equalsIgnoreCase("config")) {
	    throw new ReservedOptionException("The keyword 'config' is reserved");
	}
	Option opt = parser.addDoubleOption(longName);
	params.put(longName, opt);
    }
    
    /**
     * @return All the parameters that don't correspond to an option.
     */
    public String[] getRemainingParameters () {
	return (parser.getRemainingArgs());
    }
    
    /**
     * Parses the command line and loads the parameters into 
     * a configuration object. If a parameter is present either in the 
     * command line and in a configuration file, the command line parameter
     * overrides those in the configuration file.
     * @param args The command line to be parsed.
     * @return The configuration.
     */
    public CAConfiguration parse (String[] args) {
	CAConfiguration conf = new CAConfiguration();
	try {                 
	   //default configuration file option
	   Option configOpt = parser.addStringOption('c',"--config");
	   
	   //parsing the command line
           parser.parse(args);
           
           //getting the parameters from the configuration file.
           String filePath = (String)parser.getOptionValue(configOpt);
           if (filePath!=null) {
    	   	CAConfigFileParser fileParser = CAConfigurator.getConfigFileParserInstance();
    	   	conf = fileParser.parse(filePath);
           }
           
           //getting the parameters from the command line.
           Iterator pIt = params.keySet().iterator();
           while (pIt.hasNext()) {
               Option curOpt = params.get((String)pIt.next());
               Object curValue = parser.getOptionValue(curOpt);
               if (curValue!=null) {
        	   conf.addParameter(curOpt.longForm(), curValue);
               }
           }
	}                                                                       
	catch ( CmdLineParser.OptionException e ) {                             
	   System.err.println(e.getMessage());                                                                                 
        } 
	return (conf);
    }
}
